# -*- coding: utf-8 -*-
# file: tests.py
# author: JinTian
# time: 16/03/2018 3:23 PM
# Copyright 2018 JinTian. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ------------------------------------------------------------------------
from .modules.vision.to_video import VideoCombiner
import sys
import numpy as np

from .modules.vision.vis_kit import draw_masks
import cv2


if __name__ == '__main__':
    a = ''
    img = cv2.imread(a)

    b = [
        [[2], [3], [4]]
    ]